@@ -2,11 +2,14 @@ from __future__ import absolute_import
 
 from inspect import getsourcefile
 from os.path import abspath
+from pathlib import Path
+from typing import Union, TYPE_CHECKING
 
-from _pytest._code.code import TerminalRepr, Traceback
+from _pytest._code.code import TerminalRepr, Traceback, ExceptionInfo
 from _pytest import fixtures
 from _pytest.fixtures import FuncFixtureInfo
 from _pytest.main import Session
+from _pytest.nodes import Collector
 from _pytest.python import Module
 import py.path
 import pytest
@@ -14,6 +17,11 @@ import pytest
 from ..example import SybilFailure
 from .. import example
 
+if TYPE_CHECKING:
+    from ..sybil import Sybil
+
+PYTEST_VERSION = tuple(int(i) for i in pytest.__version__.split('.'))
+
 example_module_path = abspath(getsourcefile(example))
 
 
@@ -44,24 +52,17 @@ class SybilItem(pytest.Item):
         # pytest fixtures dance:
         fm = self.session._fixturemanager
         closure = fm.getfixtureclosure(names, self)
-        try:
-            initialnames, names_closure, arg2fixturedefs = closure
-        except ValueError:  # pragma: no cover
-            # pytest < 3.7
-            names_closure, arg2fixturedefs = closure
-            fixtureinfo = FuncFixtureInfo(names, names_closure, arg2fixturedefs)
-        else:
-            # pyest >= 3.7
-            fixtureinfo = FuncFixtureInfo(names, initialnames, names_closure, arg2fixturedefs)
+        initialnames, names_closure, arg2fixturedefs = closure
+        fixtureinfo = FuncFixtureInfo(names, initialnames, names_closure, arg2fixturedefs)
         self._fixtureinfo = fixtureinfo
         self.funcargs = {}
-        self._request = fixtures.FixtureRequest(self)
+        self._request = fixtures.FixtureRequest(self, _ispytest=True)
 
     def reportinfo(self):
         info = '%s line=%i column=%i' % (
             self.fspath.basename, self.example.line, self.example.column
         )
-        return py.path.local(self.example.document.path), self.example.line, info
+        return py.path.local(self.example.path), self.example.line, info
 
     def getparent(self, cls):
         if cls is Module:
@@ -70,7 +71,7 @@ class SybilItem(pytest.Item):
             return self.session
 
     def setup(self):
-        fixtures.fillfixtures(self)
+        self._request._fillfixtures()
         for name, fixture in self.funcargs.items():
             self.example.namespace[name] = fixture
 
@@ -78,34 +79,31 @@ class SybilItem(pytest.Item):
         self.example.evaluate()
 
     def _prunetraceback(self, excinfo):
-        # Messier than it could be because slicing a list subclass in
-        # Python 2 returns a list, not an instance of the subclass.
         tb = excinfo.traceback.cut(path=example_module_path)
         tb = tb[1]
         if getattr(tb, '_rawentry', None) is not None:
             excinfo.traceback = Traceback(tb._rawentry, excinfo)
 
-    def repr_failure(self, excinfo):
+    def repr_failure(
+        self,
+        excinfo: ExceptionInfo[BaseException],
+        style = None,
+    ) -> Union[str, TerminalRepr]:
         if isinstance(excinfo.value, SybilFailure):
             return SybilFailureRepr(self, str(excinfo.value))
-        return super(SybilItem, self).repr_failure(excinfo)
+        return super().repr_failure(excinfo, style)
 
 
 class SybilFile(pytest.File):
 
-    def __init__(self, fspath, parent, sybil):
-        super(SybilFile, self).__init__(fspath, parent)
-        self.sybil = sybil
+    def __init__(self, *, sybil: 'Sybil', **kwargs):
+        super(SybilFile, self).__init__(**kwargs)
+        self.sybil: 'Sybil' = sybil
 
     def collect(self):
-        self.document = self.sybil.parse(self.fspath.strpath)
+        self.document = self.sybil.parse(Path(self.fspath.strpath))
         for example in self.document:
-            try:
-                from_parent = SybilItem.from_parent
-            except AttributeError:
-                yield SybilItem(self, self.sybil, example)
-            else:
-                yield from_parent(self, sybil=self.sybil, example=example)
+            yield SybilItem.from_parent(self, sybil=self.sybil, example=example)
 
     def setup(self):
         if self.sybil.setup:
@@ -116,15 +114,13 @@ class SybilFile(pytest.File):
             self.sybil.teardown(self.document.namespace)
 
 
-def pytest_integration(sybil, class_=SybilFile):
+def pytest_integration(*sybils: 'Sybil'):
 
-    def pytest_collect_file(parent, path):
-        if sybil.should_test_path(path):
-            try:
-                from_parent = class_.from_parent
-            except AttributeError:
-                return class_(path, parent, sybil)
-            else:
-                return from_parent(parent, fspath=path, sybil=sybil)
+    def pytest_collect_file(path: py.path.local, parent: Collector):
+        fspath = path
+        path = Path(fspath.strpath)
+        for sybil in sybils:
+            if sybil.should_parse(path):
+                return SybilFile.from_parent(parent, path=path, sybil=sybil)
 
     return pytest_collect_file
